import sys
import os
from PyQt5.QtWidgets import QWidget, QApplication
from PyQt5.QtWidgets import QVBoxLayout
from PyQt5.QtWidgets import QTextEdit
from PyQt5.QtCore import Qt


class AboutWidget(QWidget):
    AUTHORS = [
        ["Aleksandr Sinitca", "Saint Petersburg Electrotechnical University \"LETI\", Russia",
            "amsinitca@etu.ru"],
        ["Mikhail Bogachev", "Saint Petersburg Electrotechnical University \"LETI\", Russia",
            "rogex@yandex.com"],
        ["Dmitry Kaplun", "Saint Petersburg Electrotechnical University \"LETI\", Russia",
            "dikaplun@etu.ru"],
        # ["Name","affiliation","e-mail"],
    ]

    def __init__(self, parent=None):
        super().__init__(parent)

        self._res_path = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), "resources")
        self.initUI()

    def initUI(self):
        self.setWindowTitle("About")
        self.main_layout = QVBoxLayout()
        #self.images_layout = QGridLayout()

        # self.main_layout.addLayout(self.images_layout)

        self.affiliation_widget = QTextEdit()
        self.affiliation_widget.setReadOnly(True)
        self.affiliation_widget.setTextInteractionFlags(
            Qt.LinksAccessibleByMouse)
        organisation = []
        for author in self.AUTHORS:
            if author[1] not in organisation:
                organisation.append(author[1])
        for author in self.AUTHORS:
            org_ind = organisation.index(author[1]) + 1
            s = "{name}<sup>{org_ind}</sup>, <a href=\"mailto:{mail}\">{mail}</a> <br> \n".format(
                name=author[0], org_ind=org_ind, mail=author[2])
            self.affiliation_widget.insertHtml(s)
        self.affiliation_widget.insertHtml("<br>")
        for ind, org in enumerate(organisation):
            ind += 1
            s = "{ind}) {org} <br>".format(ind=ind, org=org)
            self.affiliation_widget.insertHtml(s)

        s = "<br><br> Licensed under GNU GPLv3 License <br> <a href=\"https://gitlab.com/digiratory/biomedimaging/bcanalyzer\">https://gitlab.com/digiratory/biomedimaging/bcanalyzer</a> <br>"
        self.affiliation_widget.insertHtml(s)
        self.affiliation_widget.setAlignment(Qt.AlignCenter)

        self.main_layout.addWidget(self.affiliation_widget)
        self.setLayout(self.main_layout)
        self.setWindowFlag(Qt.Dialog and Qt.MSWindowsFixedSizeDialogHint)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    app = AboutWidget()
    app.show()
    qapp.exec_()



from re import L
from xmlrpc.client import boolean
from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import QWidget
from PyQt5.QtWidgets import QSlider, QDoubleSpinBox, QLabel
from PyQt5.QtWidgets import QHBoxLayout


class ExtendedSliderWidget(QWidget):
    valueChanged = pyqtSignal(float)

    def __init__(self, text="", parent=None) -> None:
        super().__init__(parent)
        self._text = text
        self.initUI()
        self.initSignals()
        self._ignore_change_signal = False
        self._is_integer = False

    def initUI(self):
        self._root_layout = QHBoxLayout()

        self.caption = QLabel(self._text)

        self.slider = QSlider()
        self.slider.setRange(0, 1000)
        self.slider.setOrientation(Qt.Horizontal)

        self.value_spinbox = QDoubleSpinBox()
        self.value_spinbox.setRange(0.0, 100.0)
        self.value_spinbox.setSingleStep(0.1)

        self._root_layout.addWidget(self.caption)
        self._root_layout.addWidget(self.slider)
        self._root_layout.addWidget(self.value_spinbox)

        self.setLayout(self._root_layout)

    def value(self):
        return self.value_spinbox.value()

    def setSingleStep(self, step_size: float):
        self.value_spinbox.setSingleStep(step_size)

    def set_integer(self, is_integer: bool):
        self._is_integer = is_integer
        if is_integer:
            self.value_spinbox.setDecimals(0)
        else:
            self.value_spinbox.setDecimals(2)

    def set_range(self, mininum: float, maximum: float):
        self.slider.setRange(int(mininum*10), int(maximum*10))
        self.value_spinbox.setRange(mininum, maximum)

    def set_value(self, new_value: float):
        self._ignore_change_signal = True
        self.value_spinbox.setValue(new_value)
        self._ignore_change_signal = True
        self.slider.setValue(int(new_value*10))

    def initSignals(self):
        self.slider.sliderMoved.connect(self.slider_change_value_handler)
        self.slider.sliderReleased.connect(self.slider_released)
        self.value_spinbox.valueChanged.connect(self.text_change_value_handler)

    def slider_released(self):
        value = self.slider.value()
        self.slider_change_value_handler(value)

    def slider_change_value_handler(self, new_value: float):
        if self._ignore_change_signal:
            self._ignore_change_signal = False
            return
        new_value = new_value / 10.0
        if self._is_integer:
            new_value = int(new_value)
        self._ignore_change_signal = True
        self.value_spinbox.setValue(new_value)
        self.valueChanged.emit(new_value)

    def text_change_value_handler(self, new_value: float):
        if self._is_integer:
            new_value = int(new_value)
        if self._ignore_change_signal:
            self._ignore_change_signal = False
            return
        self._ignore_change_signal = True
        self.slider.setValue(int(new_value))
        self.valueChanged.emit(new_value)




import cv2
import numpy as np

from bisect import bisect_left


def approximateContours(threshold):

    contours, _ = cv2.findContours(
        threshold, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE
    )

    if not contours:
        print('not contours')
        return np.zeros(threshold.shape, dtype=np.uint8)

    contours.sort(key=cv2.contourArea)

    epsilon = 0.001*cv2.arcLength(contours[-1], True)
    approx_contours = []

    for cnt in contours:
        approx_contours.append(
            cv2.approxPolyDP(cnt, epsilon, True)
        )

    blank_mask = np.zeros(threshold.shape, dtype=np.uint8)
    cv2.fillPoly(blank_mask, approx_contours, (255, 255, 255))

    return blank_mask


def getLargestContour(thresh):
    # RETURNS: largest contour and its area

    contours, _ = cv2.findContours(
        thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE
    )

    if contours:
        cnt = max(contours, key=cv2.contourArea)
        return cnt, cv2.contourArea(cnt)

    else:
        return 0, 0


def getLargestContourThresh(thresh):
    '''
    RETURNS: removes all contours except the largest
    '''

    blank_mask = np.zeros(thresh.shape, dtype=np.uint8)
    largest_cnt, area = getLargestContour(thresh)

    if area:
        cv2.fillPoly(blank_mask, [largest_cnt], (255, 255, 255))

    return blank_mask


def invertByMask(img, mask):
    # RETURNS: image with mask filtered inverted color areas

    if mask.max() == 1:
        mask = 255*mask
    inverted = cv2.bitwise_not(img)
    foreground = cv2.bitwise_or(inverted, inverted, mask=mask)

    mask = cv2.bitwise_not(mask)
    background = cv2.bitwise_or(img, img, mask=mask)

    final = cv2.bitwise_or(foreground, background)

    return final


def showDifference(img, reference_mask, compared_mask):
    # RETURNS: markes by invertion difference between two masks on image

    new_mask = cv2.bitwise_xor(reference_mask, compared_mask)
    new_mask = cv2.bitwise_not(new_mask)

    return invertByMask(img, new_mask)


# ---------------------{ utilites }---------------------

def getMaxValueIndex(arr):
    'use only on 1D numpy arrays'
    if not arr.size:
        print('in function \'getMaxValueIndexes\': wrong input ')
        raise ValueError

    max_value = max(arr)
    result = np.where(arr == max_value)

    return result[0][0]  # will return left largest index


def find_ge(arr, x):
    'Find leftmost item greater than or equal to x'
    i = bisect_left(arr, x)
    if i != len(arr):
        return arr[i]
    raise ValueError



import os

from PyQt5.QtCore import Qt
from PyQt5.QtCore import QCoreApplication
from PyQt5.QtCore import pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QDesktopWidget, QApplication, QMainWindow
from PyQt5.QtWidgets import QSizePolicy
from PyQt5.QtWidgets import QFileDialog, QProgressDialog, QSplitter
from PyQt5.QtWidgets import QSizePolicy, QProgressDialog
from PyQt5.QtGui import QIcon

from bcanalyzer.common.settings import AppSettings
from bcanalyzer.gui.widgets.about_widget import AboutWidget

from bcanalyzer.gui.widgets.processing_preview_widget import ProcessingPreviewWidget
from bcanalyzer.gui.widgets.processing_list_widget import ProcessinListWidget
from bcanalyzer.common.const import APP_NAME


class Main_GUI(QMainWindow):
    onClose = pyqtSignal()

    export_all = pyqtSignal(str, str)
    export_renders = pyqtSignal(str)
    export_stat = pyqtSignal(str)

    def __init__(self):
        super().__init__()
        self._title = APP_NAME
        self.setupUi()
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setAcceptDrops(True)
        self._pd = None

    def setupUi(self):
        """constructs the GUI  """
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

        self.setWindowTitle(self._title)

        ### creating the menu ###
        self._menu_bar = self.menuBar()

        self._menu_file = self._menu_bar.addMenu("File")

        self._submenu_export = self._menu_file.addMenu("Export")

        self.submenu_export_all = self._submenu_export.addAction(
            "All")
        self.submenu_export_all.triggered.connect(
            self.submenu_export_all_clicked)

        self.submenu_export_renders = self._submenu_export.addAction(
            "Renders")
        self.submenu_export_renders.triggered.connect(
            self.submenu_export_renders_clicked)

        self.submenu_export_stat = self._submenu_export.addAction(
            "Statistic")
        self.submenu_export_stat.triggered.connect(
            self.submenu_export_stat_clicked)

        self._menu_file.addSeparator()

        action = self._menu_file.addAction("Exit")
        action.triggered.connect(self.exit_clicked)

        self._menu_help = self._menu_bar.addMenu("Help")
        self._menu_help.addSeparator()
        action = self._menu_help.addAction("About")
        action.triggered.connect(self.about_clicked)

        ### create status bar ###

        self.status_bar = self.statusBar()
        self.status_bar.showMessage('Ready')

        ### Widgets ###
        self._main_widget = QSplitter(self)
        self._main_widget.setSizePolicy(
            QSizePolicy.Expanding, QSizePolicy.Expanding)

        # Main Layout
        self.processing_list_widget = ProcessinListWidget(self)

        self._main_widget.addWidget(self.processing_list_widget)

        self.processing_widget = ProcessingPreviewWidget()
        self._main_widget.addWidget(self.processing_widget)

        self.setCentralWidget(self._main_widget)
        # Icon
        res_path = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), "..", "resources")
        QApplication.instance().setWindowIcon(
            QIcon(os.path.join(res_path, "logo.ico")))

    def set_processing_list_model(self, processing_list_model):
        self.processing_list_widget.set_processing_list_model(
            processing_list_model)

    def show_progress_dialog(self, labelText, minimum, maximum, cancelButtonText="Cancel"):
        self._pd = QProgressDialog(labelText,
                                   cancelButtonText,
                                   minimum,
                                   maximum,
                                   self)
        self._pd.setValue(0)
        self._pd.setWindowModality(Qt.WindowModal)

        return self._pd

    def select_folder_dialog_launch(self, question="Select folder"):
        chosen_dir = QFileDialog.getExistingDirectory(None, "Select folder",
                                                      (AppSettings().getParam('last_open_export_dir', "./")))
        if os.path.exists(chosen_dir):
            AppSettings().setParam('last_open_export_dir', chosen_dir)
            return chosen_dir
        return None

    def select_target_file_dialog_launch(self, question="Save report"):
        file_path = QFileDialog.getSaveFileName(None, question,
                                                AppSettings().getParam('last_open_export_dir', "./"),
                                                "CSV-file (*.csv)")[0]
        if os.path.exists(os.path.dirname(file_path)):
            AppSettings().setParam('last_open_export_dir', os.path.dirname(file_path))
        return file_path

    @pyqtSlot()
    def exit_clicked(self):
        self.onClose.emit()
        self.deleteLater()
        self.destroy(True, True)
        QApplication.closeAllWindows()
        QCoreApplication.instance().quit()

    @pyqtSlot()
    def about_clicked(self):
        self.aboutWidget = AboutWidget()
        self.aboutWidget.show()

    @pyqtSlot()
    def submenu_export_all_clicked(self):
        chosen_dir = self.select_folder_dialog_launch()
        chosen_file = self.select_target_file_dialog_launch()
        if chosen_dir is not None and chosen_file:
            self.export_all.emit(chosen_dir, chosen_file)

    @pyqtSlot()
    def submenu_export_renders_clicked(self):
        chosen_dir = self.select_folder_dialog_launch()
        if chosen_dir is not None:
            self.export_renders.emit(chosen_dir)

    @pyqtSlot()
    def submenu_export_stat_clicked(self):
        chosen_file = self.select_target_file_dialog_launch()
        if chosen_file:
            self.export_stat.emit(chosen_file)



import os
import sys
import json

from bcanalyzer.common.singleton import Singleton
from bcanalyzer.common.files import config_file_path
from bcanalyzer.common.const import *


DEFAUL_SETTINGS = {
}


class AppSettings(object, metaclass=Singleton):
    """
    Class for the settings.
    """
    __settings = None

    def __init__(self):
        """
        Inits AppSettings class.
        """
        pass

    def getParam(self, key: str, default_value=None):
        """
        Get parameter from setting dictionary based on given key.

        Parameters
        ----------
        key: Key of parameter.
        default_value: Default value of the parameter.

        Returns
        -------
        The requested parameter.

        """
        if self.__settings is None:
            self._load_setting()
        if key not in self.__settings.keys():
            self.setParam(key, default_value)
        print(key, self.__settings.get(key, default_value))
        return self.__settings.get(key, default_value)

    def setParam(self, key: str, value, force=True):
        """
        Set given parameter inside setting dictionary.

        Parameters
        ----------
        key: Key of parameter.
        value: Value of parameter.
        force: Flag to force the addition.

        Returns
        -------
        True if the parameter was set, false otherwise.

        """
        if self.__settings is None:
            self._load_setting()
        if key in self.__settings.keys() or force:
            self.__settings[key] = value
            self._save_setting()
            return True
        else:
            return False

    def _load_setting(self):
        """
        Loads a settings configurations from file.

        """
        settigs_filename = AppSettings.get_settings_filename()
        if not os.path.exists(settigs_filename):
            self._create_default_settings_file()
        try:
            with open(settigs_filename, 'r', encoding='utf-8') as file:
                self.__settings = json.load(file)
        except:
            print("Error: some troubles with configuration")
        pass

    def _create_default_settings_file(self):
        """
        Creates a settings file, and saves the settings into it as json.

        """
        settigs_filename = AppSettings.get_settings_filename()
        os.makedirs(os.path.dirname(settigs_filename), exist_ok=True)
        with open(settigs_filename, 'w', encoding='utf-8') as file:
            json.dump(DEFAUL_SETTINGS, file, indent=2, sort_keys=True)

    def _save_setting(self):
        """
        Saves the settings into a file.

        """
        settigs_filename = AppSettings.get_settings_filename()
        os.makedirs(os.path.dirname(settigs_filename), exist_ok=True)
        with open(settigs_filename, 'w', encoding='utf-8') as file:
            json.dump(self.__settings, file, indent=2, sort_keys=True)

    @staticmethod
    def get_settings_filename():
        """
        Static method to get the file name of the settings file.

        """
        return config_file_path()


if __name__ == "__main__":
    sys.path.append(os.path.abspath(os.path.join(
        os.path.dirname(__file__), '..', '..')))
    print(AppSettings.get_settings_filename())
    model_uri = AppSettings().getParam("model_uri")
    print(model_uri)




from abc import ABC, abstractmethod

from PyQt5.QtCore import pyqtSignal, QObject


class MetaAbstractView(type(ABC), type(QObject)):
    pass


class AbstractController(ABC, QObject, metaclass=MetaAbstractView):
    # signal to interact with other windows
    next_window_signal = pyqtSignal(int)

    @abstractmethod
    def run(self):
        raise NotImplementedError
